@@ -1,6 +1,7 @@
 from django import forms
 from django.contrib.gis.gdal import GDALException
 from django.contrib.gis.geos import GEOSException, GEOSGeometry
+from django.core.exceptions import ValidationError
 from django.utils.translation import gettext_lazy as _
 
 from .widgets import OpenLayersWidget
@@ -47,7 +48,7 @@ class GeometryField(forms.Field):
                 except (GEOSException, ValueError, TypeError):
                     value = None
             if value is None:
-                raise forms.ValidationError(self.error_messages['invalid_geom'], code='invalid_geom')
+                raise ValidationError(self.error_messages['invalid_geom'], code='invalid_geom')
 
         # Try to set the srid
         if not value.srid:
@@ -71,14 +72,14 @@ class GeometryField(forms.Field):
         # Ensuring that the geometry is of the correct type (indicated
         # using the OGC string label).
         if str(geom.geom_type).upper() != self.geom_type and not self.geom_type == 'GEOMETRY':
-            raise forms.ValidationError(self.error_messages['invalid_geom_type'], code='invalid_geom_type')
+            raise ValidationError(self.error_messages['invalid_geom_type'], code='invalid_geom_type')
 
         # Transforming the geometry if the SRID was set.
         if self.srid and self.srid != -1 and self.srid != geom.srid:
             try:
                 geom.transform(self.srid)
             except GEOSException:
-                raise forms.ValidationError(
+                raise ValidationError(
                     self.error_messages['transform_error'], code='transform_error')
 
         return geom
@@ -89,7 +90,7 @@ class GeometryField(forms.Field):
         try:
             data = self.to_python(data)
             initial = self.to_python(initial)
-        except forms.ValidationError:
+        except ValidationError:
             return True
 
         # Only do a geographic comparison if both values are available
