@@ -140,7 +140,8 @@ def intword(value):
     except (TypeError, ValueError):
         return value
 
-    if value < 1000000:
+    abs_value = abs(value)
+    if abs_value < 1000000:
         return value
 
     def _check_for_i18n(value, float_formatted, string_formatted):
@@ -156,10 +157,10 @@ def intword(value):
 
     for exponent, converters in intword_converters:
         large_number = 10 ** exponent
-        if value < large_number * 1000:
+        if abs_value < large_number * 1000:
             new_value = value / large_number
             rounded_value = round_away_from_one(new_value)
-            return _check_for_i18n(new_value, *converters(rounded_value))
+            return _check_for_i18n(new_value, *converters(abs(rounded_value)))
     return value
 
 
