@@ -1,14 +1,14 @@
 from django.middleware.csrf import get_token
 from django.utils.functional import lazy
 from django.utils.html import format_html
-from django.utils.safestring import SafeText
+from django.utils.safestring import SafeString
 
 
 def csrf_input(request):
     return format_html(
-        '<input type="hidden" name="csrfmiddlewaretoken" value="{}" />',
+        '<input type="hidden" name="csrfmiddlewaretoken" value="{}">',
         get_token(request))
 
 
-csrf_input_lazy = lazy(csrf_input, SafeText, str)
+csrf_input_lazy = lazy(csrf_input, SafeString, str)
 csrf_token_lazy = lazy(get_token, str)
